@@ -6,7 +6,7 @@
 # -----------------------------------------------------------------------------------------------------------------------
 
 """
-Authors: Evan Hubinger, Fred Buchanan
+Authors: Evan Hubinger, Fred Buchanan, Noah Lipsyc, Ishaan Verma
 License: Apache 2.0
 Description: The Coconut command-line utility.
 """
@@ -22,21 +22,23 @@ from coconut.root import *  # NOQA
 import sys
 import os
 import time
-import traceback
+import shutil
 from contextlib import contextmanager
 from subprocess import CalledProcessError
 
+from coconut._pyparsing import (
+    unset_fast_pyparsing_reprs,
+    collect_timing_info,
+    print_timing_info,
+)
+
 from coconut.compiler import Compiler
 from coconut.exceptions import (
     CoconutException,
     CoconutInternalException,
 )
-from coconut.terminal import (
-    logger,
-    printerr,
-)
+from coconut.terminal import logger
 from coconut.constants import (
-    univ_open,
     fixpath,
     code_exts,
     comp_ext,
@@ -49,10 +51,21 @@ from coconut.constants import (
     coconut_run_args,
     coconut_run_verbose_args,
     verbose_mypy_args,
+    default_mypy_args,
     report_this_text,
-    mypy_non_err_prefixes,
+    mypy_silent_non_err_prefixes,
+    mypy_silent_err_prefixes,
+    mypy_err_infixes,
+    mypy_install_arg,
+    mypy_builtin_regex,
+    coconut_pth_file,
+)
+from coconut.util import (
+    printerr,
+    univ_open,
+    ver_tuple_to_str,
+    install_custom_kernel,
 )
-from coconut.kernel_installer import install_custom_kernel
 from coconut.command.util import (
     writefile,
     readfile,
@@ -70,9 +83,13 @@ from coconut.command.util import (
     launch_tutorial,
     stdin_readable,
     set_recursion_limit,
-    canparse,
+    can_parse,
+    invert_mypy_arg,
+)
+from coconut.compiler.util import (
+    should_indent,
+    get_target_info_smart,
 )
-from coconut.compiler.util import should_indent, get_target_info_len2
 from coconut.compiler.header import gethash
 from coconut.command.cli import arguments, cli_version
 
@@ -91,13 +108,14 @@ class Command(object):
     exit_code = 0  # exit status to return
     errmsg = None  # error message to display
     mypy_args = None  # corresponds to --mypy flag
+    argv_args = None  # corresponds to --argv flag
 
     def __init__(self):
         """Create the CLI."""
         self.prompt = Prompt()
 
     def start(self, run=False):
-        """Process command-line arguments."""
+        """Endpoint for coconut and coconut-run."""
         if run:
             args, argv = [], []
             # for coconut-run, all args beyond the source file should be wrapped in an --argv
@@ -105,13 +123,12 @@ class Command(object):
                 arg = sys.argv[i]
                 args.append(arg)
                 # if arg is source file, put everything else in argv
-                if not arg.startswith("-") and canparse(arguments, args[:-1]):
+                if not arg.startswith("-") and can_parse(arguments, args[:-1]):
                     argv = sys.argv[i + 1:]
                     break
-            if "--verbose" in args:
-                args = list(coconut_run_verbose_args) + args
-            else:
-                args = list(coconut_run_args) + args
+            for run_arg in (coconut_run_verbose_args if "--verbose" in args else coconut_run_args):
+                if run_arg not in args:
+                    args.append(run_arg)
             self.cmd(args, argv=argv)
         else:
             self.cmd()
@@ -123,6 +140,8 @@ class Command(object):
         else:
             parsed_args = arguments.parse_args(args)
         if argv is not None:
+            if parsed_args.argv is not None:
+                raise CoconutException("cannot pass --argv/--args when using coconut-run (coconut-run interprets any arguments after the source file as --argv/--args)")
             parsed_args.argv = argv
         self.exit_code = 0
         with self.handling_exceptions():
@@ -140,7 +159,7 @@ class Command(object):
         """Exit if exit_code is abnormal."""
         if self.exit_code:
             if self.errmsg is not None:
-                logger.show("Exiting due to " + self.errmsg + ".")
+                logger.show("Exiting with error: " + self.errmsg)
                 self.errmsg = None
             if self.using_jobs:
                 kill_children()
@@ -148,15 +167,29 @@ class Command(object):
 
     def use_args(self, args, interact=True, original_args=None):
         """Handle command-line arguments."""
-        logger.quiet, logger.verbose = args.quiet, args.verbose
-        if DEVELOP:
-            logger.tracing = args.trace
+        # fix args
+        if not DEVELOP:
+            args.trace = args.profile = False
+
+        # set up logger
+        logger.quiet, logger.verbose, logger.tracing = args.quiet, args.verbose, args.trace
+        if args.trace or args.profile:
+            unset_fast_pyparsing_reprs()
+        if args.profile:
+            collect_timing_info()
 
         logger.log(cli_version)
         if original_args is not None:
             logger.log("Directly passed args:", original_args)
         logger.log("Parsed args:", args)
 
+        # validate general command args
+        if args.mypy is not None and args.line_numbers:
+            logger.warn("extraneous --line-numbers argument passed; --mypy implies --line-numbers")
+        if args.site_install and args.site_uninstall:
+            raise CoconutException("cannot --site-install and --site-uninstall simultaneously")
+
+        # process general command args
         if args.recursion_limit is not None:
             set_recursion_limit(args.recursion_limit)
         if args.jobs is not None:
@@ -167,77 +200,70 @@ class Command(object):
             self.prompt.set_style(args.style)
         if args.history_file is not None:
             self.prompt.set_history_file(args.history_file)
-        if args.documentation:
+        if args.vi_mode:
+            self.prompt.vi_mode = True
+        if args.docs:
             launch_documentation()
         if args.tutorial:
             launch_tutorial()
+        if args.site_uninstall:
+            self.site_uninstall()
+        if args.site_install:
+            self.site_install()
+        if args.argv is not None:
+            self.argv_args = list(args.argv)
 
+        # additional validation after processing
+        if args.profile and self.jobs != 0:
+            raise CoconutException("--profile incompatible with --jobs {jobs}".format(jobs=args.jobs))
+
+        # process general compiler args
         self.setup(
             target=args.target,
             strict=args.strict,
             minify=args.minify,
-            line_numbers=args.line_numbers,
+            line_numbers=args.line_numbers or args.mypy is not None,
             keep_lines=args.keep_lines,
             no_tco=args.no_tco,
             no_wrap=args.no_wrap,
         )
 
+        # process mypy args (must come after compiler setup)
         if args.mypy is not None:
             self.set_mypy_args(args.mypy)
 
-        if args.argv is not None:
-            sys.argv = [args.source if args.source is not None else ""]
-            sys.argv.extend(args.argv)
-
         if args.source is not None:
+            # warnings if source is given
             if args.interact and args.run:
                 logger.warn("extraneous --run argument passed; --interact implies --run")
             if args.package and self.mypy:
                 logger.warn("extraneous --package argument passed; --mypy implies --package")
 
+            # errors if source is given
             if args.standalone and args.package:
                 raise CoconutException("cannot compile as both --package and --standalone")
             if args.standalone and self.mypy:
                 raise CoconutException("cannot compile as both --package (implied by --mypy) and --standalone")
             if args.no_write and self.mypy:
                 raise CoconutException("cannot compile with --no-write when using --mypy")
-            if (args.run or args.interact) and os.path.isdir(args.source):
-                if args.run:
-                    raise CoconutException("source path must point to file not directory when --run is enabled")
-                if args.interact:
-                    raise CoconutException("source path must point to file not directory when --run (implied by --interact) is enabled")
-            if args.watch and os.path.isfile(args.source):
-                raise CoconutException("source path must point to directory not file when --watch is enabled")
 
-            if args.dest is None:
-                if args.no_write:
-                    dest = False  # no dest
-                else:
-                    dest = True  # auto-generate dest
-            elif args.no_write:
-                raise CoconutException("destination path cannot be given when --no-write is enabled")
-            else:
-                dest = args.dest
-
-            source = fixpath(args.source)
-
-            if args.package or self.mypy:
-                package = True
-            elif args.standalone:
-                package = False
-            else:
-                # auto-decide package
-                if os.path.isfile(source):
-                    package = False
-                elif os.path.isdir(source):
-                    package = True
-                else:
-                    raise CoconutException("could not find source path", source)
+            # process all source, dest pairs
+            src_dest_package_triples = [
+                self.process_source_dest(src, dst, args)
+                for src, dst in (
+                    [(args.source, args.dest)]
+                    + (getattr(args, "and") or [])
+                )
+            ]
 
+            # do compilation
             with self.running_jobs(exit_on_error=not args.watch):
-                filepaths = self.compile_path(source, dest, package, run=args.run or args.interact, force=args.force)
+                filepaths = []
+                for source, dest, package in src_dest_package_triples:
+                    filepaths += self.compile_path(source, dest, package, run=args.run or args.interact, force=args.force)
             self.run_mypy(filepaths)
 
+        # validate args if no source is given
         elif (
             args.run
             or args.no_write
@@ -247,7 +273,10 @@ class Command(object):
             or args.watch
         ):
             raise CoconutException("a source file/folder must be specified when options that depend on the source are enabled")
+        elif getattr(args, "and"):
+            raise CoconutException("--and should only be used for extra source/dest pairs, not the first source/dest pair")
 
+        # handle extra cli tasks
         if args.code is not None:
             self.execute(self.comp.parse_block(args.code))
         got_stdin = False
@@ -263,14 +292,61 @@ class Command(object):
                 or args.source
                 or args.code
                 or args.tutorial
-                or args.documentation
+                or args.docs
                 or args.watch
+                or args.site_uninstall
+                or args.site_install
                 or args.jupyter is not None
+                or args.mypy == [mypy_install_arg]
             )
         ):
             self.start_prompt()
         if args.watch:
-            self.watch(source, dest, package, args.run, args.force)
+            # src_dest_package_triples is always available here
+            self.watch(src_dest_package_triples, args.run, args.force)
+        if args.profile:
+            print_timing_info()
+
+    def process_source_dest(self, source, dest, args):
+        """Determine the correct source, dest, package mode to use for the given source, dest, and args."""
+        # determine source
+        processed_source = fixpath(source)
+
+        # validate args
+        if (args.run or args.interact) and os.path.isdir(processed_source):
+            if args.run:
+                raise CoconutException("source path %r must point to file not directory when --run is enabled" % (source,))
+            if args.interact:
+                raise CoconutException("source path %r must point to file not directory when --run (implied by --interact) is enabled" % (source,))
+        if args.watch and os.path.isfile(processed_source):
+            raise CoconutException("source path %r must point to directory not file when --watch is enabled" % (source,))
+
+        # determine dest
+        if dest is None:
+            if args.no_write:
+                processed_dest = False  # no dest
+            else:
+                processed_dest = True  # auto-generate dest
+        elif args.no_write:
+            raise CoconutException("destination path cannot be given when --no-write is enabled")
+        else:
+            processed_dest = dest
+
+        # determine package mode
+        if args.package or self.mypy:
+            package = True
+        elif args.standalone:
+            package = False
+        else:
+            # auto-decide package
+            if os.path.isfile(source):
+                package = False
+            elif os.path.isdir(source):
+                package = True
+            else:
+                raise CoconutException("could not find source path", source)
+
+        return processed_source, processed_dest, package
 
     def register_error(self, code=1, errmsg=None):
         """Update the exit code."""
@@ -295,9 +371,9 @@ class Command(object):
             self.register_error(err.code)
         except BaseException as err:
             if isinstance(err, CoconutException):
-                logger.display_exc()
+                logger.print_exc()
             elif not isinstance(err, KeyboardInterrupt):
-                traceback.print_exc()
+                logger.print_exc()
                 printerr(report_this_text)
             self.register_error(errmsg=err.__class__.__name__)
 
@@ -364,7 +440,10 @@ class Command(object):
                 if force:
                     logger.warn("found destination path with " + dest_ext + " extension; compiling anyway due to --force")
                 else:
-                    raise CoconutException("found destination path with " + dest_ext + " extension; aborting compilation", extra="pass --force to override")
+                    raise CoconutException(
+                        "found destination path with " + dest_ext + " extension; aborting compilation",
+                        extra="pass --force to override",
+                    )
         self.compile(filepath, destpath, package, force=force, **kwargs)
         return destpath
 
@@ -391,7 +470,7 @@ class Command(object):
             if self.show:
                 print(foundhash)
             if run:
-                self.execute_file(destpath)
+                self.execute_file(destpath, argv_source_path=codepath)
 
         else:
             logger.show_tabulated("Compiling", showpath(codepath), "...")
@@ -409,7 +488,7 @@ class Command(object):
                     if destpath is None:
                         self.execute(compiled, path=codepath, allow_show=False)
                     else:
-                        self.execute_file(destpath)
+                        self.execute_file(destpath, argv_source_path=codepath)
 
             if package is True:
                 self.submit_comp_job(codepath, callback, "parse_package", code, package_level=package_level)
@@ -435,7 +514,8 @@ class Command(object):
             else:
                 break
         if package_level < 0:
-            logger.warn("missing __init__" + code_exts[0] + " in package", check_dir)
+            if self.comp.strict:
+                logger.warn("missing __init__" + code_exts[0] + " in package", check_dir, extra="remove --strict to dismiss")
             package_level = 0
         return package_level
         return 0
@@ -480,7 +560,7 @@ class Command(object):
     @property
     def using_jobs(self):
         """Determine whether or not multiprocessing is being used."""
-        return self.jobs != 0
+        return self.jobs is None or self.jobs > 1
 
     @contextmanager
     def running_jobs(self, exit_on_error=True):
@@ -533,7 +613,11 @@ class Command(object):
 
     def start_prompt(self):
         """Start the interpreter."""
-        logger.show("Coconut Interpreter:")
+        logger.show(
+            "Coconut Interpreter v{co_ver}:".format(
+                co_ver=VERSION,
+            ),
+        )
         logger.show("(enter 'exit()' or press Ctrl-D to end)")
         self.start_running()
         while self.running:
@@ -570,29 +654,48 @@ class Command(object):
         try:
             return self.comp.parse_block(code)
         except CoconutException:
-            logger.display_exc()
+            logger.print_exc()
         return None
 
     def execute(self, compiled=None, path=None, use_eval=False, allow_show=True):
         """Execute compiled code."""
         self.check_runner()
         if compiled is not None:
+
             if allow_show and self.show:
                 print(compiled)
-            if path is not None:  # path means header is included, and thus encoding must be removed
+
+            if path is None:  # header is not included
+                if not self.mypy:
+                    no_str_code = self.comp.remove_strs(compiled)
+                    result = mypy_builtin_regex.search(no_str_code)
+                    if result:
+                        logger.warn("found mypy-only built-in " + repr(result.group(0)) + "; pass --mypy to use mypy-only built-ins at the interpreter")
+
+            else:  # header is included
                 compiled = rem_encoding(compiled)
+
             self.runner.run(compiled, use_eval=use_eval, path=path, all_errors_exit=path is not None)
+
             self.run_mypy(code=self.runner.was_run_code())
 
-    def execute_file(self, destpath):
+    def execute_file(self, destpath, **kwargs):
         """Execute compiled file."""
-        self.check_runner()
+        self.check_runner(**kwargs)
         self.runner.run_file(destpath)
 
-    def check_runner(self, set_up_path=True):
+    def check_runner(self, set_sys_vars=True, argv_source_path=""):
         """Make sure there is a runner."""
-        if set_up_path and os.getcwd() not in sys.path:
-            sys.path.append(os.getcwd())
+        if set_sys_vars:
+            # set sys.path
+            if os.getcwd() not in sys.path:
+                sys.path.append(os.getcwd())
+
+            # set sys.argv
+            if self.argv_args is not None:
+                sys.argv = [argv_source_path] + self.argv_args
+
+        # set up runner
         if self.runner is None:
             self.runner = Runner(self.comp, exit=self.exit_runner, store=self.mypy)
 
@@ -605,22 +708,39 @@ class Command(object):
         """Set MyPy arguments."""
         if mypy_args is None:
             self.mypy_args = None
+
+        elif mypy_install_arg in mypy_args:
+            if mypy_args != [mypy_install_arg]:
+                raise CoconutException("'--mypy install' cannot be used alongside other --mypy arguments")
+            stub_dir = set_mypy_path()
+            logger.show_sig("Successfully installed MyPy stubs into " + repr(stub_dir))
+            self.mypy_args = None
+
         else:
-            self.mypy_errs = []
             self.mypy_args = list(mypy_args)
 
-            if not any(arg.startswith("--python-version") for arg in mypy_args):
+            if not any(arg.startswith("--python-version") for arg in self.mypy_args):
                 self.mypy_args += [
                     "--python-version",
-                    ".".join(str(v) for v in get_target_info_len2(self.comp.target, mode="nearest")),
+                    ver_tuple_to_str(get_target_info_smart(self.comp.target, mode="mypy")),
                 ]
 
-            if logger.verbose:
-                for arg in verbose_mypy_args:
-                    if arg not in self.mypy_args:
-                        self.mypy_args.append(arg)
+            if not any(arg.startswith("--python-executable") for arg in self.mypy_args):
+                self.mypy_args += [
+                    "--python-executable",
+                    sys.executable,
+                ]
+
+            add_mypy_args = default_mypy_args + (verbose_mypy_args if logger.verbose else ())
+
+            for arg in add_mypy_args:
+                no_arg = invert_mypy_arg(arg)
+                arg_prefixes = (arg,) + ((no_arg,) if no_arg is not None else ())
+                if not any(arg.startswith(arg_prefixes) for arg in self.mypy_args):
+                    self.mypy_args.append(arg)
 
             logger.log("MyPy args:", self.mypy_args)
+            self.mypy_errs = []
 
     def run_mypy(self, paths=(), code=None):
         """Run MyPy with arguments."""
@@ -628,19 +748,23 @@ class Command(object):
             set_mypy_path()
             from coconut.command.mypy import mypy_run
             args = list(paths) + self.mypy_args
-            if code is not None:
+            if code is not None:  # interpreter
                 args += ["-c", code]
             for line, is_err in mypy_run(args):
-                if line.startswith(mypy_non_err_prefixes):
-                    if code is not None:
-                        print(line)
-                else:
+                logger.log("[MyPy]", line)
+                if line.startswith(mypy_silent_err_prefixes):
+                    if code is None:  # file
+                        printerr(line)
+                        self.register_error(errmsg="MyPy error")
+                elif not line.startswith(mypy_silent_non_err_prefixes):
+                    if code is None:  # file
+                        printerr(line)
+                        if any(infix in line for infix in mypy_err_infixes):
+                            self.register_error(errmsg="MyPy error")
                     if line not in self.mypy_errs:
-                        printerr(line)
+                        if code is not None:  # interpreter
+                            printerr(line)
                         self.mypy_errs.append(line)
-                    elif code is None:
-                        printerr(line)
-                    self.register_error(errmsg="MyPy error")
 
     def run_silent_cmd(self, *args):
         """Same as run_cmd$(show_output=logger.verbose)."""
@@ -656,8 +780,8 @@ class Command(object):
             try:
                 self.run_silent_cmd(user_install_args)
             except CalledProcessError:
-                logger.warn("kernel install failed on command'", " ".join(install_args))
-                self.register_error(errmsg="Jupyter error")
+                logger.warn("kernel install failed on command", " ".join(install_args))
+                self.register_error(errmsg="Jupyter kernel error")
                 return False
         return True
 
@@ -667,13 +791,14 @@ class Command(object):
         try:
             self.run_silent_cmd(remove_args)
         except CalledProcessError:
-            logger.warn("kernel removal failed on command'", " ".join(remove_args))
-            self.register_error(errmsg="Jupyter error")
+            logger.warn("kernel removal failed on command", " ".join(remove_args))
+            self.register_error(errmsg="Jupyter kernel error")
             return False
         return True
 
     def install_default_jupyter_kernels(self, jupyter, kernel_list):
         """Install icoconut default kernels."""
+        logger.show_sig("Installing Jupyter kernels '" + "', '".join(icoconut_default_kernel_names) + "'...")
         overall_success = True
 
         for old_kernel_name in icoconut_old_kernel_names:
@@ -686,7 +811,9 @@ class Command(object):
             overall_success = overall_success and success
 
         if overall_success:
-            logger.show_sig("Successfully installed Jupyter kernels: " + ", ".join((icoconut_custom_kernel_name,) + icoconut_default_kernel_names))
+            return icoconut_default_kernel_names
+        else:
+            return []
 
     def get_jupyter_kernels(self, jupyter):
         """Get the currently installed Jupyter kernels."""
@@ -706,27 +833,31 @@ class Command(object):
             ["jupyter"],
         ):
             try:
-                self.run_silent_cmd([sys.executable, "-m", "jupyter", "--version"])
+                self.run_silent_cmd(jupyter + ["--help"])  # --help is much faster than --version
             except CalledProcessError:
-                logger.warn("failed to find Jupyter command at " + str(jupyter))
+                logger.warn("failed to find Jupyter command at " + repr(" ".join(jupyter)))
             else:
                 break
 
         # get a list of installed kernels
         kernel_list = self.get_jupyter_kernels(jupyter)
+        newly_installed_kernels = []
 
-        # always update the custom kernel, but only reinstall it if it isn't already there
-        custom_kernel_dir = install_custom_kernel()
-        if icoconut_custom_kernel_name not in kernel_list:
-            self.install_jupyter_kernel(jupyter, custom_kernel_dir)
+        # always update the custom kernel, but only reinstall it if it isn't already there or given no args
+        custom_kernel_dir = install_custom_kernel(logger=logger)
+        if custom_kernel_dir is not None and (icoconut_custom_kernel_name not in kernel_list or not args):
+            logger.show_sig("Installing Jupyter kernel {name!r}...".format(name=icoconut_custom_kernel_name))
+            if self.install_jupyter_kernel(jupyter, custom_kernel_dir):
+                newly_installed_kernels.append(icoconut_custom_kernel_name)
 
         if not args:
             # install default kernels if given no args
-            self.install_default_jupyter_kernels(jupyter, kernel_list)
+            newly_installed_kernels += self.install_default_jupyter_kernels(jupyter, kernel_list)
+            run_args = None
 
         else:
             # use the custom kernel if it exists
-            if icoconut_custom_kernel_name in kernel_list:
+            if icoconut_custom_kernel_name in kernel_list or icoconut_custom_kernel_name in newly_installed_kernels:
                 kernel = icoconut_custom_kernel_name
 
             # otherwise determine which default kernel to use and install them if necessary
@@ -739,8 +870,8 @@ class Command(object):
                 else:
                     kernel = "coconut_py" + ver
                 if kernel not in kernel_list:
-                    self.install_default_jupyter_kernels(jupyter, kernel_list)
-                logger.warn("could not find 'coconut' kernel; using " + repr(kernel) + " kernel instead")
+                    newly_installed_kernels += self.install_default_jupyter_kernels(jupyter, kernel_list)
+                logger.warn("could not find {name!r} kernel; using {kernel!r} kernel instead".format(name=icoconut_custom_kernel_name, kernel=kernel))
 
             # pass the kernel to the console or otherwise just launch Jupyter now that we know our kernel is available
             if args[0] == "console":
@@ -748,42 +879,73 @@ class Command(object):
             else:
                 run_args = jupyter + args
 
+        if newly_installed_kernels:
+            logger.show_sig("Successfully installed Jupyter kernels: '" + "', '".join(newly_installed_kernels) + "'")
+
+        # run the Jupyter command
+        if run_args is not None:
             self.register_error(run_cmd(run_args, raise_errs=False), errmsg="Jupyter error")
 
-    def watch(self, source, write=True, package=True, run=False, force=False):
-        """Watch a source and recompiles on change."""
+    def watch(self, src_dest_package_triples, run=False, force=False):
+        """Watch a source and recompile on change."""
         from coconut.command.watch import Observer, RecompilationWatcher
 
-        source = fixpath(source)
+        for src, _, _ in src_dest_package_triples:
+            logger.show()
+            logger.show_tabulated("Watching", showpath(src), "(press Ctrl-C to end)...")
 
-        logger.show()
-        logger.show_tabulated("Watching", showpath(source), "(press Ctrl-C to end)...")
-
-        def recompile(path):
+        def recompile(path, src, dest, package):
             path = fixpath(path)
             if os.path.isfile(path) and os.path.splitext(path)[1] in code_exts:
                 with self.handling_exceptions():
-                    if write is True or write is None:
-                        writedir = write
+                    if dest is True or dest is None:
+                        writedir = dest
                     else:
-                        # correct the compilation path based on the relative position of path to source
+                        # correct the compilation path based on the relative position of path to src
                         dirpath = os.path.dirname(path)
-                        writedir = os.path.join(write, os.path.relpath(dirpath, source))
+                        writedir = os.path.join(dest, os.path.relpath(dirpath, src))
                     filepaths = self.compile_path(path, writedir, package, run=run, force=force, show_unchanged=False)
                     self.run_mypy(filepaths)
 
-        watcher = RecompilationWatcher(recompile)
         observer = Observer()
-        observer.schedule(watcher, source, recursive=True)
+        watchers = []
+        for src, dest, package in src_dest_package_triples:
+            watcher = RecompilationWatcher(recompile, src, dest, package)
+            observer.schedule(watcher, src, recursive=True)
+            watchers.append(watcher)
 
         with self.running_jobs():
             observer.start()
             try:
                 while True:
                     time.sleep(watch_interval)
-                    watcher.keep_watching()
+                    for wcher in watchers:
+                        wcher.keep_watching()
             except KeyboardInterrupt:
                 logger.show_sig("Got KeyboardInterrupt; stopping watcher.")
             finally:
                 observer.stop()
                 observer.join()
+
+    def get_python_lib(self):
+        """Get current Python lib location."""
+        from distutils import sysconfig  # expensive, so should only be imported here
+        return fixpath(sysconfig.get_python_lib())
+
+    def site_install(self):
+        """Add Coconut's pth file to site-packages."""
+        python_lib = self.get_python_lib()
+
+        shutil.copy(coconut_pth_file, python_lib)
+        logger.show_sig("Added %s to %s." % (os.path.basename(coconut_pth_file), python_lib))
+
+    def site_uninstall(self):
+        """Remove Coconut's pth file from site-packages."""
+        python_lib = self.get_python_lib()
+        pth_file = os.path.join(python_lib, os.path.basename(coconut_pth_file))
+
+        if os.path.isfile(pth_file):
+            os.remove(pth_file)
+            logger.show_sig("Removed %s from %s." % (os.path.basename(coconut_pth_file), python_lib))
+        else:
+            raise CoconutException("failed to find %s file to remove" % (os.path.basename(coconut_pth_file),))
